# MT::Plugins::Redland::References
# -----------------
# $Revision: 1.6 $
# $Date: 2005/04/06 15:09:45 $
# ---------------------------------------------------------

=head1 NAME

MT::Plugins::Redland::References - Adds dcterms:references statements to the Redland model

=cut

package MT::Plugins::Redland::References;

=head1 SYNOPSIS

New triples will be added to the Redland model using dcterms:references
for every hyperlink present in the body of an entry.

=head1 REQUIRES

 MovableType 2.6 or higher
 HTML::Parser

=head1 DESCRIPTION

This plugin uses the dcterms:references predicate to link an
entry's RDF resource with those it references in HTML. See
L<http://web.resource.org/rss/1.0/modules/dcterms/#references>. 

All HTML anchors containing an href attribute are parsed from the
current entry's text, and added as dcterms:references statements.
For example, given the HTML:

  I like <a href="http://www.megatokyo.com/">MegaTokyo</a>.

the following RDF statement will be added to the model:

  {[http://example.com/ilike.html], [http://purl.org/dc/terms/references], [http://www.megatokyo.com/]}

=cut

use strict;
use warnings;


use Data::Dumper;
use RDF::Redland;
use HTML::Parser;

use MT 2.6;
use MT::Template::Context;


our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	our $NAME			= "Redland References";
	$VERSION			= do { my @REV = split(/\./, (qw$Revision: 1.6 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug				= 0;
	$plugin_data		= {
			name		=> "${NAME}, v${VERSION}",
			description => 'Adds dcterms:references statements to the Redland model',
			doc_link	=> 'http://kasei.us/code/mtredland/'
		};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
}

######################################################################

our $plugin;
my $cfg		= MT::ConfigMgr->instance;
if ($cfg->ObjectDriver eq 'Redland') {
	MT::Entry->add_callback("pre_save", 6, $plugin, sub {
		my ($cb, $entry)	= @_;
		my $html	= $entry->text;
		my $model	= $entry->driver->model;
		my @urls;
		my $parser	= HTML::Parser->new(
						api_version	=> 3,
						start_h		=> [
							sub {
								my ($tag, $attrs)	= @_;
								return unless $tag eq 'a';
								return unless exists($attrs->{'href'});
								push(@urls,$attrs->{'href'});
							},
							"tagname, attr"
						]
					);
		$parser->parse( $html );
		
		my $node	= $entry->get_rdf_node;
		my $ref		= RDF::Redland::Node->new_from_uri( 'http://purl.org/dc/terms/references' );
		my $st		= RDF::Redland::Statement->new( $node, $ref, undef );
		my @st		= $model->find_statements( $st );
		$model->remove_statement( $_ ) for (@st);
		foreach my $url (@urls) {
			$url	=~ s/&/&amp;/g;
			my $url		= RDF::Redland::Node->new_from_uri( $url );
			$st->object( $url );
			$model->add_statement( $st );
		}
	});
}

######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_references.pl,v $
 Revision 1.6  2005/04/06 15:09:45  greg
 - bumped version number to 0.005
 - updated email address in AUTHOR POD section

 Revision 1.5  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.4  2005/04/03 23:58:17  greg
 - added CVS Log POD
 - removed "see also" pod from reviews plugin that were copied from references plugin


=head1 SEE ALSO

 L<MT::Plugin>
 L<HTML::Parser>
 L<http://web.resource.org/rss/1.0/modules/dcterms/#references>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <greg@evilfunhouse.com>

=cut
